@@ -1,16 +1,39 @@
-from django.core.cache import get_cache
+import socket
+import warnings
+
+from django.core.cache import get_cache, parse_backend_uri
 from django.utils.datastructures import SortedDict
 
 import nexus
 
 from nexus_memcache import conf
 
-cache = get_cache(conf.BACKEND)._cache
+def get_caches():
+    caches = []
+    schema, hosts, params = parse_backend_uri(conf.BACKEND)
+    for host in hosts.split(';'):
+        caches.append((host, get_cache('%s://%s?%s' % (schema, host, params))._cache))
+    return caches
+caches = get_caches()
 
 class MemcacheModule(nexus.NexusModule):
     home_url = 'index'
     name = 'memcache'
     
+    def get_stats(self, timeout=5):
+        for host, cache in caches:
+            default_timeout = socket.getdefaulttimeout()
+            socket.setdefaulttimeout(timeout)
+            try:
+                stats = cache.get_stats()[0][1]
+            except:
+                stats = {'online': 0}
+            else:
+                stats['online'] = 1
+            finally:
+                socket.setdefaulttimeout(default_timeout)
+            yield host, stats
+    
     def get_title(self):
         return 'Memcache'
     
@@ -25,12 +48,12 @@ class MemcacheModule(nexus.NexusModule):
     
     def render_on_dashboard(self, request):
         try:
-            cache_stats = cache.get_stats()
+            cache_stats = list(self.get_stats())
         except AttributeError:
+            warnings.warn('`get_stats()` not found on cache backend')
             cache_stats = []
         
         global_stats = {
-            'accepting_conns': 0,
             'bytes': 0,
             'limit_maxbytes': 0,
             'curr_items': 0,
@@ -41,10 +64,11 @@ class MemcacheModule(nexus.NexusModule):
             'get_hits': 0,
             'get_misses': 0,
             'rusage_system': 0,
+            'online': 0,
         }
         for host, stats in cache_stats:
             for k in global_stats.iterkeys():
-                global_stats[k] += float(stats[k])
+                global_stats[k] += float(stats.get(k, 0))
         global_stats['total'] = len(cache_stats)
 
         return self.render_to_string('nexus/memcache/dashboard.html', {
@@ -53,7 +77,7 @@ class MemcacheModule(nexus.NexusModule):
     
     def index(self, request):
         try:
-            cache_stats = ((k, SortedDict(sorted(v.iteritems(), key=lambda x: x[0]))) for k, v in cache.get_stats())
+            cache_stats = ((k, SortedDict(sorted(v.iteritems(), key=lambda x: x[0]))) for k, v in self.get_stats())
         except AttributeError:
             cache_stats = []
         
